@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Niklas Fiekas <niklas.fiekas@backscattering.de>
+// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -83,11 +83,11 @@ fn init_magics(sq: Square, magic: &Magic, shift: u32, attacks: &mut [Bitboard],
 fn dump_slice<W: Write, T: LowerHex>(
     w: &mut W,
     name: &str,
-    tname: &str,
+    table_name: &str,
     slice: &[T],
 ) -> io::Result<()> {
     writeln!(w, "#[allow(clippy::unreadable_literal)]")?;
-    write!(w, "static {}: [{}; {}] = [", name, tname, slice.len())?;
+    write!(w, "static {}: [{}; {}] = [", name, table_name, slice.len())?;
     for v in slice {
         write!(w, "0x{:x}, ", v)?;
     }
@@ -102,7 +102,7 @@ fn dump_table<W: Write, T: LowerHex>(
 ) -> io::Result<()> {
     writeln!(w, "#[allow(clippy::unreadable_literal)]")?;
     write!(w, "static {}: [[{}; 64]; 64] = [", name, tname)?;
-    for row in table.iter() {
+    for row in table {
         write!(w, "[")?;
         for column in row {
             write!(w, "0x{:x}, ", column)?;
