@@ -1,5 +1,5 @@
 // This file is part of the shakmaty library.
-// Copyright (C) 2017-2021 Niklas Fiekas <niklas.fiekas@backscattering.de>
+// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -477,8 +477,10 @@ impl Fen {
     /// # Ok::<_, Box<dyn std::error::Error>>(())
     /// ```
     pub fn from_ascii(fen: &[u8]) -> Result<Fen, ParseFenError> {
-        let mut parts = fen.split(|ch| *ch == b' ' || *ch == b'_');
         let mut result = Fen::empty();
+        let mut parts = fen
+            .split(|ch| *ch == b' ' || *ch == b'_')
+            .filter(|s| !s.is_empty());
 
         let board_part = parts.next().expect("splits have at least one part");
 
@@ -524,23 +526,35 @@ impl Fen {
         match parts.next() {
             Some(b"-") | None => (),
             Some(castling_part) => {
-                for &ch in castling_part {
-                    let color = Color::from_white(ch < b'a'); // uppercase
+                result.castling_rights = castling_part
+                    .iter()
+                    .map(|ch| {
+                        let color = Color::from_white(ch.is_ascii_uppercase());
+                        let rooks_and_kings = result.board.by_color(color)
+                            & (result.board.rooks() | result.board.kings())
+                            & color.backrank();
+                        Ok(match ch.to_ascii_lowercase() {
+                            b'k' => rooks_and_kings
+                                .last()
+                                .filter(|sq| result.board.rooks().contains(*sq))
+                                .unwrap_or_else(|| Square::from_coords(File::H, color.backrank())),
+                            b'q' => rooks_and_kings
+                                .first()
+                                .filter(|sq| result.board.rooks().contains(*sq))
+                                .unwrap_or_else(|| Square::from_coords(File::A, color.backrank())),
+                            file => Square::from_coords(
+                                File::from_char(char::from(file))
+                                    .ok_or(ParseFenError::InvalidCastling)?,
+                                color.backrank(),
+                            ),
+                        })
+                    })
+                    .collect::<Result<_, ParseFenError>>()?;
 
-                    let candidates = result.board.by_piece(color.rook()) & color.backrank();
-
-                    let flag = match ch | 32 {
-                        b'k' => candidates.last(),
-                        b'q' => candidates.first(),
-                        file @ b'a'..=b'h' => {
-                            (candidates & File::new(u32::from(file as u8 - b'a'))).first()
-                        }
-                        _ => return Err(ParseFenError::InvalidCastling),
-                    };
-
-                    result
-                        .castling_rights
-                        .add(flag.ok_or(ParseFenError::InvalidCastling)?);
+                for color in Color::ALL {
+                    if (result.castling_rights & color.backrank()).count() > 2 {
+                        return Err(ParseFenError::InvalidCastling);
+                    }
                 }
             }
         }
@@ -659,7 +673,19 @@ mod tests {
     #[test]
     fn test_invalid_fen() {
         assert!(
-            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQQKBNR w cq - 0P1"
+            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQQKBNR w cq - 0P1" // syntax
+                .parse::<Fen>()
+                .is_err()
+        );
+
+        assert!(
+            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w  - 0 1" // double space
+                .parse::<Fen>()
+                .is_err()
+        );
+
+        assert!(
+            "4k2r/8/8/8/8/8/8/RR2K2R w KBQk - 0 1" // triple castling rights
                 .parse::<Fen>()
                 .is_err()
         );
